print <<'EOF';
<html>
<body text="#f5deb3" link="#f5de83" alink="#b3ae71" vlink="#b3ae71" bgcolor="#2f4f4f">

How to translate Haskell type classes to merd types?

First some remarks:
<ul>
<li>merd's syntax for classes may change:
   <ul>
     <li>the magic "<tt>o</tt>" is expanded to "<tt>o !&lt; current_class</tt>"
     <li>the use of "<tt>:=</tt>" may be replaced by the more <i>classical</i> "<tt>!!</tt>"
     <li>no syntax is available to declare to methods with the same type
   </ul>
<li>note the confusion between classes and types in Haskell for the "Ratio" example

<li>telling merd that something is an instance of a type-class is not
necessary. It is used to tell the compiler to check that this something is
effectively an instance of that type-class.

<li>I took these examples from the Haskell Prelude. These examples do not
correspond in any way to the merd library. The only non-necessary adaptations
to suit the merd-style are the use of an uncurried syntax, and the replacement
of "<tt>[a]</tt>" by "<tt>List(a)</tt>"

</ul>


<p>
EOF


my $s = '
class Eq a where
    (==), (/=) :: a -> a -> Bool

Eq = class
    ::== := o,o -> Bool
    ::/= := o,o -> Bool


class (Eq a) => Ord a where
    compare :: a -> a -> Ordering

Ord = Eq |&| class
    compare := o,o -> Ordering


class (Real a, Fractional a) => RealFrac a where
    ...

RealFrac = Real |&| Fractional |&| class
    ...


instance Eq Char where
    ...

...
Char !< Eq


instance Eq a => Eq [a] where
    ...

...
List(Eq) !< Eq


instance Integral a => Ord (Ratio a) where
    ...

...
Ratio(Integral) !< Ord


class Collection e ce | ce -> e where
    empty :: ce
    insert :: e -> ce -> ce
    member :: e -> ce -> Bool

Collection(e) = class
    empty := o(e)
    insert := e, o(e) -> o(e)
    member := e, o(e) -> Bool


instance Eq a => Collection a (a -> Bool) where
    empty = (\x -> False)
    insert e f = (\x -> if x == e then True else f x)
    member e f = f e

empty = x -> False
insert(e,f) = x -> if x == e then True else f(x)
member(e,f) = f(e)
e -> (e !< Eq) ; (e -> Bool) !< Collection


class Collection e ce => Sequence e ce where
    cons :: e -> ce -> ce

Sequence(e) = Collection(e) |&| class
    cons := e, o(e) -> o(e)


class (Eq a, Eq b, Collection (a,b) cab)
  => Association a b cab | cab -> a, cab -> b
  where
    keys :: cab -> [a]
    values :: cab -> [b]
    has_key :: a -> cab -> Bool

Association(a,b) =
  a !< Eq
  b !< Eq
  Collection(a,b) |&| class
    keys := o(a,_) -> List(a)
    values := o(_,b) -> List(b)
    has_key := a, o(a,_) -> Bool


instance Sequence a [a] where
    ...

...
List !< Sequence


instance Sequence Int [Int] where
    ...

...
a -> (a !< Int) ; List(a) !< Sequence


instance Num a => Sequence a [a] where
    ...

...
a -> (a !< Num) ; List(a) !< Sequence


instance Sequence Bool String where
    single e = if e then "1" else "0"

single(e) = if e then "1" else "0"
Bool -> String !< Sequence
# more readable: String !< Sequence(Bool)


instance (Eq a, Eq b)
  => Association a b (Assoc a b) where
    ...

...
Assoc(Eq, Eq) !< Association
';

$\ = "\n";
print "<table border=1 cellpadding=4>";
foreach (split "\n\n\n", $s) {
    my ($haskell, $merd) = my @l = split "\n\n";
    @l == 2 or die "bad text $_";
    print "<tr>", join('', map { "<td><pre>\n" . html_quote($_) . "\n</pre></td>" } $haskell, $merd), "</tr>";
}
print "</table>\n";

print <<'EOF';
<hr>
<address><a href="mailto:pixel@merd.net">Pixel</a></address>
This document is licensed under <a href="http://www.gnu.org/copyleft/fdl.html">GFDL</a> (GNU Free Documentation License).
<p>Release: $Id: type-classes-haskell2merd.html.pl,v 1.4 2002/05/17 15:22:57 pixel_ Exp $

<p><A HREF="http://www.estat.com/getstats?serial=1400113423"><IMG SRC="http://perso.estat.com/cgi-bin/perso/1400113423?page=merd" BORDER=0></A>

</body>
</html>
EOF

sub html_quote {
    local ($_) = @_;
    s/</&lt;/g;
    s/>/&gt;/g;
    $_;
}
