open List
open Common


(* dumb version, need enhancement.
   the goal is to minimize the number of comparison

   see http://citeseer.nj.nec.com/ait-kaci89efficient.html ?
*)

type 'a lattice = ('a * 'a bounds) list
and 'a bounds = { lower : 'a list ; upper : 'a list }

let rec lower_than latt (e, { lower = l }) = e :: collect (fun a -> lower_than latt (a, assq a latt)) l
let rec upper_than latt (e, { upper = l }) = e :: collect (fun a -> upper_than latt (a, assq a latt)) l

let lower_bounds latt = filter_some (function e, { lower = [] } -> Some e | _ -> None) latt
let upper_bounds latt = filter_some (function e, { upper = [] } -> Some e | _ -> None) latt

let rec lower_bounds_ latt e =
  match assq e latt with
  | { lower = [] } -> [e]
  | { lower = l } -> collect (fun a -> lower_bounds_ latt a) l

let rec upper_bounds_ latt e = 
  match assq e latt with
  | { upper = [] } -> [e]
  | { upper = l } -> collect (fun a -> upper_bounds_ latt a) l

let compute_bounds_ compare latt e =
  let rec compute_bounds_rec current_bounds = function
    | [] -> `Bounds current_bounds
    | (e', _) as elem :: l ->
	try
	  (match compare e' e with
	  | 0 -> `Already_in_lattice e'
	  | -1 ->
	      (* it is greater than "elem", no need to test with lower values *)
	      let closed_lower = lower_than latt elem in
	      let (_l_lower, l') = partition (fun (a,_) -> memq a closed_lower) l in

	      (* remove unneeded info (un-closure) *)
	      let lower = difference_ (==) current_bounds.lower closed_lower in
	      compute_bounds_rec { current_bounds with lower = e' :: lower } l'
	  | _ (*1*) ->
	      (* it is lower than "elem", no need to test with upper values *)
	      let closed_upper = upper_than latt elem in
	      let (_l_upper, l') = partition (fun (a,_) -> memq a closed_upper) l in

	      (* remove unneeded info (un-closure) *)
	      let upper = difference_ (==) current_bounds.upper closed_upper in
	      compute_bounds_rec { current_bounds with upper = e' :: upper } l')
        with Not_comparable ->
	  (* try with something else *)
	  compute_bounds_rec current_bounds l
  in
  compute_bounds_rec { lower = [] ; upper = [] } latt
  

let create() = []
let add_ compare latt e =
  match compute_bounds_ compare latt e with
  | `Bounds bounds ->
      let latt2 = fold_left (fun latt a -> update_assoc_by (==) (fun bounds' -> 
	{ bounds' with upper = e :: difference_ (==) bounds'.upper bounds.upper } 
      ) a latt) latt  bounds.lower in
      let latt3 = fold_left (fun latt a -> update_assoc_by (==) (fun bounds' -> 
	{ bounds' with lower = e :: difference_ (==) bounds'.lower bounds.lower }
      ) a latt) latt2 bounds.upper in
      e, (e, bounds) :: latt3
  | `Already_in_lattice e' -> e', latt

let add compare latt e = snd (add_ compare latt e)

let compute_bounds compare latt e =
  match compute_bounds_ compare latt e with
  | `Bounds bounds -> bounds
  | `Already_in_lattice e' -> { lower = [e'] ; upper = [e'] } (* exact match *)

let least_upper_than compare latt e = 
  let bounds = compute_bounds compare latt e in bounds.upper
let greatest_lower_than compare latt e = 
  let bounds = compute_bounds compare latt e in bounds.lower

let greatest_upper_than compare latt e =
  let upper = least_upper_than compare latt e in
  uniq_ (==) (collect (upper_bounds_ latt) upper)

let test() =
  let stringify_one e = String.concat "|" (map string_of_int e) in
  let stringify_list l = String.concat ", " (map stringify_one l) in
  let print_elem (e, bounds) = Printf.printf "(%s) < %s < (%s)\n" (stringify_list bounds.lower) (stringify_one e) (stringify_list bounds.upper) in
  let print_lattice latt = iter print_elem latt in

  let elems = [
    [ 1 ] ;
    [ 1 ; 2 ; 3 ; 4 ; 5 ] ;
    [ 1 ; 3 ] ;
    [ 1 ; 2 ; 3 ] ;
    [ 1 ; 2 ; 3 ; 4 ] ;
    [ 1 ; 2 ; 3 ; 5 ] ;
    [ 6 ] ;
    [ 1 ; 2 ] ;
    [ 7 ] ;
    [ 7 ; 8 ] ;
  ] in

  (*
              1|2|3|4|5
                /   \
	  1|2|3|4   1|2|3|5
                \   /
                1|2|3
                /   \
              1|2   1|3
                \   /
                 \ /
                  1 
   *)

  let latt = fold_left (fun latt e ->
    let latt' = add compare latt e in
    print_lattice latt' ; print_string "********************************************************************************\n"; print_newline();
    latt'
  ) (create()) elems in

  let bound_it e =
    let verb_is_greater a b =
      Printf.printf "comparing %s and %s\n" (stringify_one a) (stringify_one b) ;
      compare a b in
    let bounds = compute_bounds verb_is_greater latt e in
    print_elem (e, bounds) ; print_string "----------\n"; print_newline();
  in
  bound_it [ 1 ; 5 ] ;
  ()
