open Types
open Print
open Rew
open List
open Common
open Package
open Printf

;;

Gc.set { (Gc.get()) with Gc.space_overhead = 70 } ;

(*
Sys.set_signal Sys.sigalrm (Sys.Signal_handle (fun _ -> failwith "timeout")) ;
ignore (Unix.alarm 10) ;
*)

;;

let isatty fd = try let _ = Unix.tcgetattr fd in true with _ -> false

let interpret pkgs =

  if not !Package.debug then (
    if not Flags.quiet then (eprintf "Starting evaluation\n"; flush stderr);

    let env = Env.new_env_ pkgs in

    (*let v' = Env.rew_remove_type_constraints v in*)

    try 
      let results = map (fun pkg -> pkg.bytecode_type, Interpreter.eval_main env pkg.bytecode) pkgs in
      let (t, v) = last results in

      let unsep = function Sep(",", [ e ]) -> e | e -> e in
      if sep2l_ (unsep v) <> [] then (
	printf "%s\n" (rew2s (unsep v)) ;
	if not (is_same v t) then printf "  !< %s\n" (rew2s (unsep t))
      )

    with Interpreter.Interpreter_rew(e) -> failwith (sprintf "user exception: %s\n" (rew2s e))
  )

let parse_options =
  let args = ref [] in
  let on_cmdline = ref None in
  let options = [
    "--give-types", Arg.Set Flags.debug_infere_and_interpret, "  print the types (verbose!)" ;
    "-e", Arg.String (fun e -> on_cmdline := Some e), "<program>  program passed in as string" ;
  ] in
  let usage = "Usage: merd [options] [-c cmd | file | -] [args]\nOptions are:" in
  Arg.parse options (lpush args) usage;

  let file_name, channel_optional, args =
    match !on_cmdline, !args with
    | Some cmdline, args -> "-", Some (Lexing.from_string cmdline), args
    | None, [] -> 
	(match Flags.default_file with
	| Some f -> f, None, []
	| None -> 
	    if isatty Unix.stdin then
	      (Arg.usage options usage; exit 1)
	    else
	      "-", Some (Lexing.from_channel stdin), [])
    | None, f :: args -> f, None, args
  in
  Interpreter.args := args ;
  file_name, channel_optional
;;

let file_name, channel_optional = parse_options
let packages = Package.doit file_name channel_optional 
;;
interpret (map snd packages)

;;1;;
