exception TooMuchRParen

type pos = string * int * int

type fromlexer = 
   | OP of string * pos
   | SPACE of pos
   | INDENT of string
   | IDENT of string * pos
   | BIGIDENT of string * pos
   | NUM of float * pos
   | STRING of string * pos
   | EOF

   | LIST of (string * pos) list * fromlexer list
   | INDENT_ of int
   | FQIDENT of string * string * pos
   | FQBIGIDENT of string * string * pos

   | CALL of fromlexer * fromlexer
   | SEP of string * fromlexer list

type fromgrouper = [
  | `Float of float * pos
  | `String of string * pos

  | `Id of string * pos
  | `Constr of string * pos
  | `Fqid of string * string * pos
  | `Fqconstr of string * string * pos
	
  | `Call of fromgrouper * fromgrouper
  | `Sep of string * fromgrouper list ]

type lazyness = Lazy | NonLazy


type where = [ `Around | `Separator | `Infix | `Prefix | `Suffix | `NotOp ]
type asso = [ `None | `Right | `Left]
type operators = (string * (int * where * asso)) list
type all_operators = { arounds_op : (string * string) list ; afters_op : operators ; befores_op : operators ; multi_ops_op : string list list }


type val_typ = fromrewrite ref ref * tconstraint * val_flags * string ref

and local_env = val_typ array array

and virtuality = Virtual | Implemented

and package = {
    full_name : string ;
    from_mec : bool ;
    bytecode : fromrewrite ; bytecode_type : fromrewrite ;
    overloading_funcs : overload_func list ;
    p_global : val_typ array ;
    operators : all_operators ;
    line_starts : int list ;
    re_exported_packages : string list ; (* full names *)
    dependencies : string list ;
    p_virtual_classes : p_virtual_classes ;
  }
      
and env = { 
    local : local_env ; 
    (*        pkg      indexed-valtyps *)
    global : (string * val_typ array) list ;
    virtual_classes : virtual_classes ;
    current_package : string ;
    overloading_funcs4pkg : overload_func list ;
  }

and overload_func = {
    overload_var : fromrewrite ;
    overload_val : fromrewrite ;
    overload_type : fromrewrite ;
    overload_is_open_type : bool ;
  } 

and virtual_classes = {
    lattice : fromrewrite Lattice.lattice ;    
    class2info : (fromrewrite * virtual_class) list ;
    class4pkg : p_virtual_classes ;
  }
and virtual_class = { 
    class_methods : (string * fromrewrite) list ;
    instanced_by : fromrewrite Lattice.lattice ;
  }

and p_virtual_classes = {
    (*              class        method *)
    p_instances : (fromrewrite * fromrewrite) list ;
    p_classes : (fromrewrite * p_virtual_class) list ;
  }
and p_virtual_class = {
    p_methods : (string * fromrewrite) list
  } 

and val_flags = { 
    para_or_local : [ `Local | `Parameter ] ; 
    is_lazy_and_alike : [ `Lazy | `Dynamic ] option ;
    is_attached_to_a_variable : bool ;
    is_constr : bool ;
    is_closed : bool ;
    is_type   : bool ;
    is_constraintable : bool ;
    pattern_matching__implying_closed_world : bool ;
    inoutness : tinoutness option 
  }

and fromrewrite = 
  | Float of float * pos
  | String of string * pos

  | Constr of string * string * pos

  | Local of (int * int * string) * pos
  | Global of (int * string * string) * pos

  | External of string * pos
  | Call of fromrewrite * fromrewrite
  | Func of int * fromrewrite list * fromrewrite
	(* nbpara para         body *)
  | Ref of fromrewrite ref * string * pos

  | Sep of string * fromrewrite list
  | SepL of string * fromrewrite list list * fromrewrite list

and tinoutness = In | Out | Exact | Inout

and tconstr = { lower : fromrewrite ; greater : fromrewrite }

and tconstraint = 
  | TFree
  | TExact of fromrewrite
  | TConstr of tconstr

type valtyp =
  | Compile_time of fromrewrite
  | Run_time of fromrewrite * fromrewrite
